package jarnik.ld22
{
	import org.flixel.FlxG;
	import org.flixel.FlxU;
	import org.flixel.FlxGroup;
	import org.flixel.FlxButton;
	import org.flixel.FlxState;
	import org.flixel.FlxObject;
	import org.flixel.FlxBasic;
	import org.flixel.FlxText;
	import org.flixel.FlxSprite;
	import org.flixel.FlxPoint;
	import org.flixel.FlxCamera;
	import org.flixel.FlxRect;

	public class PlayState extends FlxState
	{

        private var animals:FlxGroup;

        private var dragging:Animal;
        private var offset:FlxPoint;
        private var finished:Boolean = false;

        public function PlayState(
        ):void {
            offset = new FlxPoint();
        }

        private function trimTemper( temper:Number ):Number {
            if ( temper > 1 )
                return temper - 1;
            if ( temper < 0 )
                return temper + 1;
            return temper;
        }

		override public function create():void
		{
            FlxG.bgColor = 0xffe0e0e0;

            add( new FlxSprite( 0,0,Assets.ImgVignette ));

            animals = new FlxGroup();
            add( animals );

            var groups:Number = 3;
            var groupSize:Number;
            var groupTemper:Number;
            var temper:Number;
            var temperAcceptRange:Number = 0.2;
            for (var g:int = 0; g < groups; g++ ) {
                groupSize = Math.floor(FlxG.random()*3+2);
                groupTemper = FlxG.random();
                for (var m:int = 0; m < groupSize; m++ ) {
                    temper = trimTemper( groupTemper + (FlxG.random()*temperAcceptRange) - temperAcceptRange/2 );
                    animals.add( new Animal( 150, 150, temper, groupSize - 1 ) );
                }
            }

            var animal:Animal;
            var margin:Number = 30;
            for each ( animal in animals.members ) {
                animal.x = margin + (FlxG.width - margin*2)*FlxG.random();
                animal.y = margin + (FlxG.height - margin*2)*FlxG.random();
            }

            alignAnimals();

            FlxG.mouse.show(Assets.ImgCursor, 2, -16, -16 );

            FlxG.play( Assets.SndStart );
            FlxG.flash( 0xffffffff, 1 );

            FlxG.playMusic( Assets.SndMusic );
		}

        private function alignAnimals():void {
            animals.sort( "y" );
            updateAnimals();
        }

        private function temperDifference( t1:Number, t2:Number ):Number {
            if ( Math.abs( t1 - t2 ) < 0.5 )  
                return Math.abs( t1 - t2 );
            return 1 - Math.abs( t1 - t2 );
        }

        private function updateAnimals():void {
            var mood:Number;
            var acceptDistance:Number = 24;
            var temperAcceptRange:Number = 0.2;
            for each ( var animal:Animal in animals.members ) {
                mood = 0;
                for each ( var other:Animal in animals.members ) {
                    if ( other == animal )
                        continue;
                    if (  
                        Math.sqrt( Math.pow( animal.x - other.x,2 ) + Math.pow( animal.y - other.y,2 ) ) > acceptDistance
                    ) 
                        continue;
                    if ( temperDifference(animal.temper, other.temper) > temperAcceptRange )
                        mood--;
                    else
                        mood++;
                }
                animal.updateMood( mood );
            }
        }

        override public function update():void {
            super.update();

            if ( finished )
                return;
              
            var animal:Animal;
            if ( !dragging ) {
                if ( FlxG.mouse.justPressed() ) {
                    // find target
                    for ( var i:int = animals.length-1; i >= 0; i-- ) {
                        animal = animals.members[i];
                        if ( 
                            FlxG.mouse.x > animal.x - 8  && FlxG.mouse.y > animal.y - 8 &&
                            FlxG.mouse.x < (animal.x + animal.width + 16) && FlxG.mouse.y < (animal.y + animal.height + 16)
                        ) {
                            dragging = animal;
                            FlxG.mouse.show(Assets.ImgCursorGrab, 2, -16, -16 );
                            FlxG.play( Assets.SndPickup );
                            break;
                        }
                    }
                    if ( dragging ) { 
                        offset.x = FlxG.mouse.x - dragging.x;
                        offset.y = FlxG.mouse.y - dragging.y;
                    }
                }
            } else {
                if ( FlxG.mouse.justReleased() ) {
                    dragging = null;
                    FlxG.play( Assets.SndPut );
                    FlxG.mouse.show(Assets.ImgCursor, 2, -16, -16 );
                } else {
                    if ( !FlxG.mouse.pressed() ) {
                        dragging = null;
                        FlxG.play( Assets.SndPut );
                        FlxG.mouse.show(Assets.ImgCursor, 2, -16, -16 );
                    } else {
                        dragging.x = FlxG.mouse.x - offset.x;
                        dragging.y = FlxG.mouse.y - offset.y;
                        alignAnimals();
                    }
                }
            }

            // check for endgame
            if ( !dragging ) {
                finished = true;
                for each ( animal in animals.members ) {
                    if ( animal.mood < 1 )
                        finished = false;
                }
                if ( finished ) {
                    FlxG.play( Assets.SndComplete );
                    add( new FlxSprite( 0,0,Assets.ImgComplete ) );
                    add( new FlxButton( 117, 111, "CONTINUE", onLevelCompleted ) );
                }
            }
        }

        private function onLevelCompleted():void {
            FlxG.play( Assets.SndClick );
            FlxG.switchState( new PlayState() );
        }

	}
}
